class SyncController < ApplicationController
  VERSION=20131217

  skip_before_filter :verify_authenticity_token

  before_filter :ensure_correct_version
  before_filter :validate_token_and_load_share, :except => [:auth]
  before_filter :ensure_local_file, :except => [:changes, :mkdir, :put, :auth]

  def auth
    if Rails.env.development?
      url = "http://localhost:3001" 
    else
      url = "http://localhost:82" 
    end

    response = RestClient::Request.new(
      :method => :get,
      :url => "#{url}/system/me",
      :user => params[:user],
      :password => params[:password],
      :headers => { 
        :accept => :json,
        :content_type => :json
      }
    ).execute

    response_json = JSON.parse(response)
    raise "user is not allowed to use protosync" unless response_json["protosync_available"]
    share = Share.find_or_create_by(path: response_json["protosync_path"], user_id: response_json["user_id"])
    params[:device] ||= "old_client"
    token = share.generate_token_for_device(params[:device])
    logger.info "==========>>>>>> #{response_json}"
    render json: {token: token}
  rescue RuntimeError => e
    logger.info "==========>>>>>> #{e.inspect}"
    render json: {token: "ERROR #{e.inspect}"}, :status => 406
  end

  def changes
    changes = @share.changeset(params[:txn]) 
    render :json => changes
  end

  def retrieve
    # TODO
    # * mtime, mode, sha1 via headers?
    # * use send_file
    send_file @local_file.path
  end

  def retrieve_patch
    send_data @local_file.delta(params[:local_signature])
  end

  def signature
    send_data @local_file.signature
  end

  def mkdir
    @local_file.mkdir
    render nothing: true
  end

  def put
    @local_file.write(params[:content], params[:mtime].to_i, params[:mode].to_i)
    render nothing: true
  end

  def rmdir
    @local_file.rmdir
    render nothing: true
  end

  def rm
    @local_file.rm
    render nothing: true
  end

  def patch
    @local_file.patch(params[:patch], params[:mtime].to_i, params[:mode].to_i)
    render nothing: true
  end

  private 

    def ensure_correct_version
      if params[:version].to_i < VERSION
        render :json => {message: "Please download new software"}, :status => 412
      end
    end

    def validate_token_and_load_share
      if token = Token.where(:token => params[:token]).take
        @share = token.share
        unless LocalFile.in_jail?(@share.path, params[:path])
          raise "trying to escape '#{@share.path}/#{params[:path]}'"  
        end
        
        @local_file = LocalFile.new("#{@share.path}/#{params[:path]}")
      else
        render :nothing => true, :status => 404
      end
    end

    def ensure_local_file
      unless @local_file.exists?
        render :nothing => true, :status => 404
      end
    end
end
