class Command < ActiveRecord::Base
  belongs_to :remote
  serialize :command

  before_save :extract_remote_txn

  def execute
    local_file = LocalFile.new(command["path"])

    $journal.info("L:#{command["command"]} #{command["path"]}") if $journal

    # puts "#{command["command"]} '#{local_file.path}'"

    case command["command"]
    when "rmdir"
      local_file.rmdir

    when "mkdir"
      local_file.mkdir 

    when "rm"
      local_file.rm

    when "ensure"
      if local_file.exists?
        unless local_file.up_to_date?(command["sha1"], command["mtime"], command["mode"])
          local_file.update_from_remote(remote, command["mtime"], command["mode"])
        end
      else
        local_file.create_from_remote(remote, command["mtime"], command["mode"])
      end
    end

    SyncFile.create_or_update_from_path(local_file.path, remote.share)        

    true
  end

  private 
    def extract_remote_txn
      self.remote_txn = command["txn"] rescue nil
    end
end