class DirectorySyncer
  def initialize(share)
    @share = share
  end

  def scan
    Dir.chdir(@share.path) do
      @share.scan_and_delete do
        read_directory(".")
      end
    end
  end

  def update_file(path, stat, directory = false)
    if sync_file = @share.sync_file_by_path(path)
      sync_file.stat = stat
      sync_file.is_directory = directory
    else
      sync_file = SyncFile.new(path: path, is_directory:directory, stat:stat, share:@share)
    end

    @share.seen_file(sync_file)

    sync_file.save!
  end

  def read_directory(path, recurse = true)
    Dir.foreach(path) do |item|
      next if item == '.' or item == '..'

      full_path = "#{path}/#{item}"

      next if SyncFile.ignored? full_path 
      next unless stat = File::Stat.new(full_path) rescue nil
      next unless stat.readable?

      if stat.directory?
        update_file(full_path, stat, true)
        read_directory(full_path) if recurse
      elsif stat.file?
        update_file(full_path, stat)
      else 
        puts "DirectorySyncer.recurse #{full_path}: file-type unsupported"
      end
    end
  end

  def changeset(from_txn = 0)
    commands = SyncFile.where("txn > ?", from_txn).order("txn asc").map do |sync_file|
      sync_file.to_command
    end
    commands
  end
end