class Share < ActiveRecord::Base
  has_many :sync_files
  has_many :tokens
  has_one :remote, dependent: :destroy

  validates :path, presence: true

  attr_accessor :status_monitor

  class << self

    def list
      Share.all.each do |share|
        puts "#{share.path} #{share.remote.token}"
      end
    end

    def scan_loop
      while true
        scan
        sleep 10
      end
    end

    def scan
      Share.all.each do |share|
        share.scan
      end
    end

    def scan_and_push
      Share.all.each do |share|
        share.apply
        share.scan
        share.remote.push
        share.remote.pull
      end
    end

    def push
      Share.all.each do |share|
        share.remote.push
      end
    end

    def pull
      Share.all.each do |share|
        share.remote.pull
      end
    end

    def reset
      Share.all.each do |share|
        share.remote.reset
      end
    end

    def apply
      Share.all.each do |share|
        share.remote.apply
      end
    end
  end

  def scan
    log_status("looking_for_changes")
    syncer = DirectorySyncer.new(self)
    syncer.scan
  end

  def apply
    Dir.chdir(path) do
      commands = remote.commands;
      inx = 1
      cnt = commands.size
      commands.order("id").each do |command|
        transaction do 
          log_status("downloading", inx, cnt)
          inx += 1
          command.destroy if command.execute
        end
      end
    end
  end

  def scan_and_delete
    transaction do 
      # TODO: load only "not deleted"
      @all_sync_files = sync_files.reload.to_a.inject ({}) { |h,i| h[i.path] = i; h }

      yield
      
      @all_sync_files.values.sort_by { |file| -file.path.size }.each do |deleted|
        deleted.mark_as_deleted
        deleted.save!
      end
    end
  end

  def next_txn
    (sync_files.maximum("txn") || 0) + 1
  end

  def seen_file(file)
    @all_sync_files.delete(file.path)
  end

  def sync_file_by_path(path)
    @all_sync_files[path]
  end

  def changeset(from_txn = 0)
    sync_files.where("txn > ?", from_txn).order("txn asc").map do |sync_file|
      sync_file.to_command
    end
  end

  def generate_token_for_device(device_mac)
    token = nil
    loop do
      token = SecureRandom.urlsafe_base64
      break token unless Token.where(token: token).first
    end
    t = Token.find_or_create_by(device_mac: device_mac, share_id: id)
    t.update_attribute(:token, token)
    token
  end

  private 
    def log_status(message, *args)
      status_monitor.call(message, *args) if status_monitor
    end
  
end
