class AppDelegate
  include BW::KVO


  def applicationWillTerminate(notification)
    p "applicationWillTerminate"
    @protosync.stop
    @share_monitor.stop
  end

  def setup_status_menu
    @app_name = NSBundle.mainBundle.infoDictionary['CFBundleDisplayName']

    @status_menu = NSMenu.new
    @status_menu.setAutoenablesItems(false)

    @status_item = NSStatusBar.systemStatusBar.statusItemWithLength(NSVariableStatusItemLength).init
    @status_item.setMenu(@status_menu)
    @status_item.setHighlightMode(true)

    @open_in_finder = @item = createMenuItem(t("Open in Finder"), 'openInFinder')
    @item.enabled = false
    @status_menu.addItem(@item)

    @item = createMenuItem(t("Open in Web"), 'openInWeb')
    # @item.enabled = false
    @status_menu.addItem(@item)

    @item = createMenuItem(t("Open Sync folder"), 'openProtosyncFolder')
    # @item.enabled = false
    @status_menu.addItem(@item)


    @status_menu.addItem(NSMenuItem.separatorItem)

    # @protosync_menu_item = @item = createMenuItem("Protosync Status", 'openProtosyncStatusWindow:')
    # @protosync_menu_item.enabled = false
    # @status_menu.addItem(@item)


    @thies = @item = createMenuItem("(hase)", nil)
    thiesoview = NSView.alloc.initWithFrame(NSMakeRect(20, 20, 250, 20))

    @protosync_status = NSTextField.alloc.initWithFrame(NSMakeRect(20, 0, 250, 20))
    @protosync_status.editable = false
    @protosync_status.font = NSFont.menuFontOfSize(14)
    @protosync_status.textColor = NSColor.colorWithDeviceRed(0, green: 0, blue: 0, alpha: 0.5)
    @protosync_status.bezeled = false
    @protosync_status.drawsBackground = false
    @protosync_status.stringValue = t("not_started_yet")
    # @protosync_status.alignment = NSCenterTextAlignment
    thiesoview.addSubview(@protosync_status)

    @item.setView thiesoview
    @status_menu.addItem(@item)

    @status_menu.addItem(NSMenuItem.separatorItem)

    @launch_at_login_menu = @item = createMenuItem(t("Launch on login"), 'launchAtLogin:')
    @status_menu.addItem(@item)

    @status_menu.addItem(NSMenuItem.separatorItem)

    @login_menu_item = @item = createMenuItem(t("Connecting to Protonet..."), 'loginLogoutAction:')

    @status_menu.addItem(@item)
    @status_menu.addItem(NSMenuItem.separatorItem)
    # @status_menu.addItem createMenuItem("About #{@app_name}", 'orderFrontStandardAboutPanel:')
    @status_menu.addItem createMenuItem(t("Check for updates..."), 'checkForUpdates')
    @status_menu.addItem createMenuItem(t("Quit"), 'terminate:')
  end

  def openInFinder
    `open #{shellescape(@share_monitor.share_local_path)}`
  end

  def openInWeb
    `open #{shellescape(@protonet_box.best_web_path)}`
  end

  def openProtosyncFolder
    `open #{shellescape(App.documents_path + "/Protosync")}`
  end

  def checkForUpdates
    @updater.checkForUpdates self
  end

  def createMenuItem(name, action)
    NSMenuItem.alloc.initWithTitle(name, action: action, keyEquivalent: '')
  end

  def startHockey
    BITHockeyManager.sharedHockeyManager.configureWithIdentifier(
      "046d62aa213922af1b1dbb6c646654fd",
      companyName:"protonet",
      delegate:self)

    BITHockeyManager.sharedHockeyManager.startManager
  end

  def share_is_now_monted_at(local_path)
    @open_in_finder.enabled = !local_path.nil?
  end

  def remove_old_launchctl
    protosync_plist = ENV['HOME'] + "/Library/LaunchAgents/protonet.sync.plist"

    if File.exists? protosync_plist
      `launchctl unload #{shellescape(protosync_plist)}`
      `rm -f #{shellescape(protosync_plist)}`
      run_alert(t("I have detected an old Protosync agent on this system and have migrated the configuration."))
    end
  end

  def applicationWillFinishLaunching(notification)
    if App.info_plist['Release'] == true
      PFMoveToApplicationsFolderIfNecessary()
    end
  end

  def launchAtLogin sender
    if LaunchAtStartup.enabled
      LaunchAtStartup.disable
    else
      LaunchAtStartup.enable
    end

    @launch_at_login_menu.checked = LaunchAtStartup.enabled
  end

  def updateLaunchOnLogin
    unless LaunchAtStartup.enabled || App::Persistence['LaunchAtStartupInstalled']
      LaunchAtStartup.enable
      App::Persistence['LaunchAtStartupInstalled'] = true
    end

    @launch_at_login_menu.checked = LaunchAtStartup.enabled
  end

  def applicationDidFinishLaunching(notification)
    remove_old_launchctl

    unless App.info_plist['Release'] == true
      # else the password of the user ends up in the log of BW:HTTP
      # BW.debug = true
    else
      startHockey
    end

    setup_status_menu

    @status_icons = IconSet.new("status")

    @status_item.setAlternateImage(@status_icons['highlighted'])

    @event_manager = EventManager.new
    @protonet_box = ProtonetBox.new

    @protosync = Protosync.new(@protonet_box)
    @share_monitor = ShareMonitor.new(@protonet_box)

    @sync_path_is_new = !Dir.exists?(@protosync.sync_path)

    @updater = SUUpdater.new
    @in_update = false

    buildWindows
    connectEvents

    updateLaunchOnLogin


    observe(@share_monitor, :share_local_path) do |old_value, new_value|
      share_is_now_monted_at(new_value)
    end

    observe(@protosync, :current_status) do |old_value, new_value|
      if @sync_path_is_new
        setProtonetFolderIconForPath @protosync.sync_path
        @sync_path_is_new = false
      end

      case new_value["action"]
      when "different_box"
        @protosync.stop rescue nil
        run_alert(t("Your Protosync folder is connected to another Box or User. Please rename your Protosync folder"))
      when "sync_not_available"
        run_alert(t("Can't use Protosync as guest!"))
        @protosync.stop rescue nil
      end

      @protosync_status.stringValue = t(new_value["action"], new_value["args"])
    end

    observe(@protonet_box, :best_web_path) do |old_value, new_value|
      @protosync.stop

      if @protonet_box.protosync_available?
        @protosync.start
      end
    end

    observe(@protonet_box, :status) do |old_value, new_value|
      # p ["status", old_value, new_value]

      if new_value == :credentials_ok || new_value == :no_protosync_available
        @status_item.setImage @status_icons["ok"]
        # animation = CABasicAnimation.animationWithKeyPath "opacity"
        # animation.duration = 0.8
        # animation.repeatCount = 100000
        # animation.autoreverses = true
        # animation.fromValue= 0.8
        # animation.toValue= 0.2
        # @status_item.layer.addAnimation animation
      else
        @status_item.setImage @status_icons["disabled"]
      end

      case new_value
      when :update_software
        updateSoftware
      when :no_credentials
        loginLogoutAction self
      when :credentials_ok
        unless @protonet_box.protosync_available?
          @protosync.current_status = {"action" => "sync_not_available"}
          # @protosync_status.stringValue = t("Sync: Not available for guests.")
        end

        closeLoginWindow
        @login_menu_item.setTitle(t("Logout"))
        # @protosync_menu_item.enabled = true
        @share_monitor.start
      when :login_failed, :login_error
        run_alert(t("Login failed."))
      end
    end

    @protonet_box.configure!
  end

  def bringToForeground
    NSApp.activateIgnoringOtherApps(true)
  end

  def loginLogoutAction sender
    bringToForeground

    if @protonet_box.status == :credentials_ok
      @share_monitor.stop
      @protosync.stop
      @protonet_box.forget_password

      # @protosync_menu_item.enabled = false

      @login_menu_item.setTitle(t("Connect to Protonet..."))
    else
      @loginWindowController.window.makeKeyAndOrderFront(self)
    end
  end

  def closeLoginWindow
    @loginWindowController.close
  end

  def updateSoftware
    return if @in_update

    @in_update = true
    @updater.checkForUpdates(self)
  end

  def protosync_alert(msg)
    @status_item.setImage @status_icons["error"]
  end

  def connectEvents
    @event_manager.listen("alert") do |params|
      protosync_alert(params["message"])
    end

    @event_manager.listen("ConnectedToDifferentBox") do |params|
      @protosync.stop rescue nil
      run_alert(t("Your Protosync folder is connected to another Box or User. Please rename your Protosync folder"))
    end

    @event_manager.listen("updateSoftware") do |params|
      updateSoftware
    end

    @event_manager.listen("open") do |params|
      `open #{shellescape(params["path"])}`
    end

    @event_manager.listen("checkAuth") do |params|
      @protosync.unload
      @loginWindowController.window.makeKeyAndOrderFront(self)
      run_alert(t("Please login to protoSync."))
    end

    @event_manager.listen("openWindow") do |params|
      case params["which"]
      when "login"
        @loginWindowController.window.makeKeyAndOrderFront(self)
      end
    end
  end

  def buildWindows
    @loginWindowController = LoginController.alloc.initWithWindowNibName('login_window')
    @loginWindowController.protonet_box = @protonet_box
  end

  def protosyncDidAuthenticate
    @loginWindowController.close
    @protosync.install_and_start_launch_agent
  end

  def protosyncLoginFailed(message)
    run_alert(message)
  end

  def t(name, args = nil)
    NSBundle.mainBundle.localizedStringForKey(name, value:nil, table:nil) % args
  end

  def setProtonetFolderIconForPath(path)
      image = NSImage.alloc.initWithContentsOfFile("resources/folder.icns")
      if image
        NSWorkspace.sharedWorkspace.setIcon(image, forFile: path, options:0)
        NSLog("Set Folder.icns for '%@'", path)
      end
  end
end
