class BackgroundProcess
  attr_reader :last_line

  def initialize(cmd, args)
    @cmd = cmd
    @args = args
  end

  def run(&data_received_callback)
    raise "already running" if running?
    @data_received_callback = data_received_callback
    # Thread.new { start_process }
    performSelectorInBackground('start_process', withObject: nil)
  end

  def stop
    Process.kill("INT", @proc.pid) if @proc
  end

  def running?
    ! @proc.nil?
  end

  def pid
    @proc && @proc.pid
  end

  def received_line(line)
    @data_received_callback.call(BW::JSON.parse(line))
  rescue Exception => e
    p line
    p e
  end

  private

  def start_process
    # p ["start", @cmd]

    @proc = IO.popen(@cmd, "r+")
    @proc.write(BW::JSON.generate(@args) + "\n")
    @proc.flush

    while true
      line = @proc.gets
      if line.nil?
        @proc.close
        @proc = nil
        raise "process #{@cmd} died"
      else
        received_line(line.strip)
      end
    end
  rescue Exception => e
    puts "stopped #{e}"
    @data_received_callback.call({ "action" => "error", "error" => e })
  end
end