class ProtonetBox
  attr_accessor :url, :user, :password, :me, :status, :best_web_path

  def initialize
    self.status = :none
  end

  def configure!
    if load_persisted_data
      start_authenticate
    else
      self.status = :no_credentials
    end
  end

  def connect(url, user, password)
    self.url      = url
    self.user     = user
    self.password = password
    start_authenticate
  end

  def software_up_to_date?
    if @me['osx_client_min_version']
      ! (Version.new(@me['osx_client_min_version']) > Version.new(App.version))
    else
      true 
    end
  end

  def protosync_available?
    @me['protosync_available']
  end

  def box_identifier
    @me['box_identifier']
  end

  def start_authenticate
    self.status = :authenticate_in_progess
    BW::HTTP.get("#{url}/system/me.json", credentials: {username: user, password: password}, allow_self_signed_certificates: true) do |response|
      if response.ok?
        @me = BW::JSON.parse(response.body.to_str)
        persist_data

        if software_up_to_date?
          self.status = :credentials_ok
        else
          self.status = :update_software
        end
      elsif response.status_code.to_s =~ /40\d/
        self.status = :login_failed
      else
        p "start_authenticate #{response.error_message}"
        self.status = :login_error
      end
    end
  rescue Exception => e
    p "start_authenticate #{e}"
    self.status = :login_error
  end

  def load_persisted_data
    self.url = App::Persistence['protonet.url']
    self.user = App::Persistence['protonet.user']
    self.password = SSKeychain.passwordForService(url, account:user)

    url && user && password
  end

  def persist_data
    App::Persistence['protonet.url'] = url
    App::Persistence['protonet.user'] = user
    
    SSKeychain.setPassword(password, forService:url, account:user)
  end

  def forget_password
    SSKeychain.deletePasswordForService(url, account:user)
    self.password = nil
    self.status = :no_credentials
  end

  def local_ip=(new_ip)
    self.best_web_path = new_ip ? "http://#{new_ip}" : url
  end
end